#include <iostream>
#include <fstream>
#include <string>
#include "camlib.h"
#include "kihon.h"

//ROOT libraries
#include "TROOT.h" 
#include "TFile.h"
#include "TNtuple.h"
#include "TString.h"
#include "TH1F.h"

using namespace std;

int main(int argc, char **argv){

  //define event number & run number
  int evn;
  if(argc == 2){
    evn = atoi(argv[1]);
  }else{
    cout << "How many events do you want to take? -->";
    cin >> evn;
    cout << endl;
  }
  int rnum = GetRunNumber();
  cout << "Event number to take: " << evn << endl << "Run number: " << rnum << endl;
  
  //define ROOTtuple
  TString filename = "root/run";filename += rnum;filename += ".root";
  TFile *f_out = new TFile(filename,"recreate");
  TString tupledata = "";
  for(int i = 1;i <= ADCsize;i++){
    tupledata += "adc";
    tupledata += i;
    tupledata += ":";
  }
  for(int i = 1;i <= TDCsize;i++){
    tupledata += "tdc";
    tupledata += i;
    tupledata += (i==TDCsize) ? "" : ":";
  }
  TNtuple *nout_org = new TNtuple("orgdata","original data",tupledata);
  TNtuple *nout_ped = new TNtuple("peddata","pedestal data",tupledata);
  TNtuple *nout_data = new TNtuple("h1","data after correction",tupledata);
  //TNtuple *nout_org = new TNtuple("orgdata","original data","evn:ch:adc:tdc");
  //TNtuple *nout_ped = new TNtuple("peddata","pedestal data","evn:ch:adc:tdc");
  //TNtuple *nout_data = new TNtuple("data","data after correction","evn:ch:adc:tdc");
  
  //camac initialization
  CAMACinit();
  MODinit();

  //event loop start
  cout << endl << endl << "event loop start!" << endl;
  string dataname_1=".kihon.dat";
  DAQ_kihon(evn,nout_org,f_out,dataname_1);
  
  //Pedestal run start
  cout << "Please turn off light switch." << endl;
  getchar();

  cout << "Pedestal run start!(1000event)" << endl;
  string dataname_2 = ".ped.dat";
  DAQ_kihon(1000,nout_ped,f_out,dataname_2);
  
  //measuring dark count
  int count[SCch];
  cout << "Dark Counting...." << endl;
  scaler(10.,count);

  //Pedestal correction
  cout << endl << "Pedestal Correction start! Converting data...." << endl;
  cout << "Pedestal cut is more than " << PedCut << "ch in TDC" << endl;
  
    //getting data from datfile
  ifstream fdata(dataname_1.c_str()),fped(dataname_2.c_str());
  int buf[ADCsize+TDCsize],ped[ADCsize][PedCut],peak[ADCsize];
  for(int i = 0;i < ADCsize;i++){
    for(int j = 0;j < PedCut;j++)ped[i][j] = 0;
  }

  //define pedestal peak
  for(int i = 0;i < 1000;i++){
    
    for(int j = 0;j < (ADCsize+TDCsize);j++){
      fped >> buf[j];
    }

    for(int k = 0;k < ADCsize;k++){
      if(buf[k]>0 && buf[k]<PedCut){
        ped[k][buf[k]]++;
      }
    }

  }
  
  for(int ch = 0;ch < ADCsize;ch++){
    int max = 0;
    for(int i = 0;i < PedCut;i++){
      if(max < ped[ch][i]){
	max = ped[ch][i];
        peak[ch] = i;
      }
    }
    cout << "ped peak ch" << ch << ": " <<peak[ch] << endl;
  }
  
  //correction ADC
  for(int i = 0;i < evn;i++){
    for(int j = 0;j < (ADCsize+TDCsize);j++){
      fdata >> buf[j];
    }

    float filldata[ADCsize+TDCsize];
    for(int ch = 0;ch < ADCsize;ch++){
      //nout_data->Fill(i,ch+1,buf[ch] - peak[ch] + 100,buf[ch + ADCsize]);
      filldata[ch] = buf[ch] - peak[ch] + 100;
    }
    for(int ch = 0;ch < TDCsize;ch++){
      filldata[ADCsize + ch] = buf[ADCsize + ch];
    }
    nout_data->Fill(filldata);
    
    if((i%50000) == 0)cout << ".";
  }
  cout << endl;
  
  cout << "....done." << endl << endl;
  fdata.close();
  fped.close();

  //termination
  CAMACterm();

  TNtuple *orgdata = (TNtuple*)f_out->FindObject("orgdata");
  orgdata->Write();
  TNtuple *peddata = (TNtuple*)f_out->FindObject("peddata");
  peddata->Write();
  //TNtuple *data = (TNtuple*)f_out->FindObject("data");
  TNtuple *data = (TNtuple*)f_out->FindObject("h1");
  data->Write();
  f_out->Write();
  f_out->Close();
  cout << "Creating roottuple '" << filename << "' has done." << endl;
  
  dataname_1 = "rm -f " + dataname_1;
  system(dataname_1.c_str());
  dataname_2 = "rm -f " + dataname_2;
  system(dataname_2.c_str());

  return 0;
}



int GetRunNumber(void){
  ifstream f("run_number");
  string name;
  f >> name;
  f.close();
  
  ofstream ofs("run_number");
  int namenum = atoi(name.c_str());
  namenum++;
  if(ofs){
    ofs << namenum ;
  }
  ofs.close();
  
  return atoi(name.c_str());
}

void MODinit(void){
  cerr << "ADC & TDC initialize...";

  int d,q,x;
  //ADC initialization
  CAMAC(NAF(ADCn,0,9), &d, &q, &x);
  CAMAC(NAF(ADCn,0,26), &d, &q, &x);
  
  //TDCinitialization]
  CAMAC(NAF(TDCn,0,9), &d, &q, &x);
  CAMAC(NAF(TDCn,0,26), &d, &q, &x);

  cerr << "finished" << endl;
}

void DAQ_kihon(int evn,TNtuple *ntout,TFile *f_out,string dataname){
  int d,q,x,timeout,*dataptr;
  int adc[ADCsize],tdc[TDCsize],tdcnum[TDCsize],savenum[ADCsize+TDCsize];

  for(int i=0;i<TDCsize;i++)tdcnum[i]=0;
  ofstream outf(dataname.c_str());

  for(int event = 1;event <= evn;event++){
    //Wait LAM loop
    do{
      CAMAC(NAF(ADCn,0,9),&d,&q,&x);
      CAMAC(NAF(TDCn,0,9),&d,&q,&x);
      
      timeout = 0;
      while((CWLAM(3) & lam) != lam){
        timeout++;
        if(timeout == 10000){
          cout << " wait lam = " << CWLAM(3) << endl;
          break;
        }
      }
    }while( timeout > 9999);
    
    //Data Read
    //ADC
    dataptr =0;
    dataptr = adc;
    for(int i = 0;i < ADCsize;i++){
      CAMAC(NAF(ADCn,ADCStart+i,0),dataptr+i,&q,&x);
    }
    //TDC
    dataptr =0;
    dataptr = tdc;
    for(int i = 0;i < TDCsize;i++){
      CAMAC(NAF(TDCn,TDCStart+i,0),dataptr+i,&q,&x);
      if(tdc[i] < PedCut)tdcnum[i]++;
    }

    //output file
    float filldata[ADCsize+TDCsize];
    for(int i = 0;i < ADCsize;i++){
      //ntout->Fill(event,i+1,adc[i],tdc[i]);
      filldata[i] = adc[i];
    }
    for(int i =0;i < TDCsize;i++){
      filldata[ADCsize + i] = tdc[i];
    }
    ntout->Fill(filldata);
    
    for(int i =0;i < ADCsize;i++){
	    outf << adc[i] << endl;
    }

    for(int i =0;i < TDCsize;i++){
	    outf << tdc[i] << endl;
    }
    
    //calling online monitor
    if(event%1000 == 0)OnlineMonitor(event,evn,tdcnum,savenum);
    
  }

  cout << endl << "DAQ has finished!" << endl << endl;
  outf.close();
}


void OnlineMonitor(int event,int evn,int *tdcnum,int *savenum){
  ifstream f("run_number");
  string rnum;
  f >> rnum;
  f.close();
  int rnum_int;
  rnum_int = atoi(rnum.c_str()) - 1;
  cout << "Run number: " << rnum_int << endl << endl;
  cout << "Event: " << event << endl;
  cout << " ******* hit events *******" << endl;
  for(int i = 0;i < TDCsize;i++){
    cout << "ch" << TDCStart+i+1 << ": " << *(tdcnum+i) <<  " , ";
  }
  cout << endl << " ******* Event rates *******" << endl;
  for(int i = 0;i < TDCsize;i++){
    cout << "ch" << TDCStart+i+1 << ": " << ( (double)*(tdcnum+i) * 100 / (double)event ) << "%" << " , ";
  }
  if(event > 1000){
    cout << endl << " ******* Defference hit number *******" << endl;
    for(int i = 0;i < TDCsize;i++){
      cout << "ch" << TDCStart+i+1 << ": " << *(tdcnum+i) - *(savenum+i) << " , ";
    }
  }
  cout << endl << endl << " This run will take more " << (evn - event) / 1000 / 60 << " min." << endl << endl;
  
  for(int i = 0;i < TDCsize;i++){
    *(savenum+i) = *(tdcnum+i);
  }
  
}




